/*
Copyright (c) 2017 European Organization for Nuclear Research (CERN).
All rights reserved. This program and the accompanying materials
are made available under the terms of the GNU Public License v3.0
which accompanies this distribution, and is available at
http://www.gnu.org/licenses/gpl.html

Contributors:
    .European Organization for Nuclear Research    (CERN) - initial API and implementation
    .GSI Helmholtzzentrum für Schwerionenforschung (GSI)  - features and bugfixes
*/


#ifndef _THREAD_H_
#define _THREAD_H_

#include <silecs-communication/interface/core/Context.h>
#include <silecs-communication/interface/utility/SilecsException.h>

namespace Silecs
{

/*! -------------------------------------------------------------------------
 * \class Task
 * \brief Template for the Task class.
 * A Task class is defined by what the user wants to execute (user_function)
 * and by the data he wants to use (user_data).
 * It is up to the user to specify the Task template according to
 * his data type.
 */
template<typename UserDataType>
class Task
{
public:
    Task(int (*pFunc)(UserDataType, Context* pContext), UserDataType data);
    int execute(Context* pContext);

private:
    int (*userFunc)(UserDataType, Context* pContext); //function user
    UserDataType userData_; //data user
};

template<typename UserDataType>
Task<UserDataType>::Task::Task(int (*pFunc)(UserDataType, Context* pContext), UserDataType data) :
                userFunc(pFunc),
                userData_(data)
{
}

template<typename UserDataType>
int Task<UserDataType>::execute(Context* pContext)
{
    return userFunc(userData_, pContext);
}
} // namespace

#endif // _THREAD_H_
